==> XMLObject-0.0.1.tar.gz: XMLObject-0.0.1/XMLObject/Node.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# XMLObject is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# ResPyre is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ResPyre; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from UserList import UserList
from types import UnicodeType
import classregistry

from main import XMLObject

def encode(v,enc = "iso-8859-1"):
    if isinstance(v, UnicodeType):
        v = v.encode(enc)
    return v


class RequiredNodeError(Exception):

    def __init__(self, name):
        self._msg = "'%s' attribute is not mandatory" % name

    def __str__(self):
        return self._msg

    
class XONode(object):
    """ This is the base class for XMLObject's content. Each node has:

        - a `type` (string)
        - an `itemType` (string) : used by ListNode so that it remembers
          what kind of items it has to store.
        - a `default` value
        - a `facultative` boolean switch

        For each of these properties, there are getters and setters.
        * `xmlrepr` is the method responsible of giving a string representing
          the node. User defined nodes should override it.
        * `getValueFromDom` is used to build a Node given its DOM tree.
    """

    def __init__(self, type='', itemType='', default=None, facultative=False):
        self.setType(type)
        self.setItemType(itemType)
        self.setValue(default)
        self.setName('')
        self.setFacultative(facultative)

    def setFacultative(self, val):
        self._facultative = val

    def isFacultative(self):
        return self._facultative

    def setItemType(self, type):
        self._itemType = type

    def getItemType(self):
        return self._itemType
        
    def setName(self, name):
        self._name = name

    def getName(self):
        return self._name

    def setValue(self, value):
        self._value = value
        
    def getValue(self):
        return self._value

    def setType(self, type):
        self._type = type

    def getType(self):
        return self._type

    def getParentType(self):
        return self._parentType

    def setParentType(self, parentType):
        self._parentType = parentType

    def xmlrepr(self, indent=0):
        return ''

    def getValueFromDom(self, dom, attrName, **kw):
        return None

    def checkType(self, val):
        return True
    
class StringAttribute(XONode):
    """
        Suppose you have an XMLObject type like this one:

        class FooBar(XMLObject):
            glop = StringAttribute()

        FooBar(glop="Hey Yah").toXml() would output:

        <FooBar glop="Hey Yah" />
    """

    def __init__(self, default='', facultative=False):
        if facultative:
            default = None
        XONode.__init__(self, type='StringAttribute',
                        default=default, facultative=facultative)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return '%s="%s"' % (self.getName(),val)
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        xmlAttrNode = dom.getAttributeNode(attrName)
        try:
            value = encode(xmlAttrNode.value)
        except AttributeError:
            if not self.isFacultative():
                raise RequiredNodeError(self.getName())
            else:
                value = None
        return value

    def checkType(self, val):
        if self.isFacultative() and val is None:
            return True
        else:
            return type(val) == type('')

class IntegerAttribute(XONode):
    """

    """
    
    def __init__(self, default=0, facultative=False):
        if facultative:
            default = None
        XONode.__init__(self, type='IntegerAttribute',
                        default=default, facultative=facultative)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return '%s="%d"' % (self.getName(), val)
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        xmlAttrNode = dom.getAttributeNode(attrName)
        if not self.isFacultative() and not xmlAttrNode:
            raise RequiredNodeError(self.getName())
        if not xmlAttrNode:
            value = None
        else:
            value = int(encode(xmlAttrNode.value))
        return value

    def checkType(self, val):
        if self.isFacultative() and val is None:
            return True
        else:
            return type(val) == type(0)

class TextNode(XONode):
    """ Example class:

        class Foo(XMLObject):
            text = TextNode()

        f = Foo(text = 'Kind of fun')
        print f.toXml(headers=False)
        
        Example XML output:

        <Foo>
            <text>
                object value as a string
            </text>
        </Foo>
    """

    def __init__(self, default = '', facultative=False):
        XONode.__init__(self, type = 'Text', facultative=facultative,
                        default = default)

    def xmlrepr(self, indent=0):
        return ' ' * indent + self.getValue() + '\n'

    def getValueFromDom(self, dom, attrName, **kw):
        value = ''
        for node in dom.childNodes:
            if node.nodeType == node.TEXT_NODE:
                value += node.data
        return encode(value.strip())

class ItemNode(XONode):
    """ Class used to refer to an existing XMLObject type.
        Suppose you have an XMLObject type like this one:

        class FooBar(XMLObject):
            glop = StringAttribute()

        You can then use it in other XMLObjects:

        class Blah(XMLObject):
            fb = ItemNode('FooBar')

        blah = Blah( fb = FooBar(glop = 'HelloWorld') )

        blah.toXml() would output:

            <Blah>
                 <FooBar glop="HelloWorld" />

            </Blah>
    """

    def __init__(self, itemType, default=None, facultative=False):
        XONode.__init__(self, type='Item', itemType=itemType,
                        facultative=facultative, default=default)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return val.toXml(indent=indent,headers=0) + '\n'
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        itemTypeName = self.getItemType()
        parentTypeName = self.getParentType()
        xmlObject = classregistry.registry(kw['registry']).getClass(itemTypeName)()
        candidates = dom.getElementsByTagName(itemTypeName)
        if len(candidates) == 0 and not self.isFacultative():
            raise RequiredNodeError(self.getName())
        value = None
        for candidate in candidates:
            if candidate.parentNode.nodeName == parentTypeName:
                value = xmlObject._fromDom(candidate)
                break
        return value

class ListNode(XONode):
    """


    """
    
    def __init__(self, itemType):
        XONode.__init__(self, type='List', itemType=itemType,
                        default=TypedList(self))
        
    def xmlrepr(self, indent=0):
        result = ''
        for node in self.getValue():
            result +=  node.toXml(indent=indent,headers=0) + '\n'
        return result

    def getValueFromDom(self, dom, attrName, **kw):
        xmlList = TypedList(self)
        itemTypeName = self.getItemType()
        parentTypeName = self.getParentType()
        klass = classregistry.registry(kw['registry']).getClass(itemTypeName)
        for xmlNode in dom.getElementsByTagName(itemTypeName):
            if xmlNode.parentNode.nodeName == parentTypeName:
                xmlObject = klass()
                xmlList.append( xmlObject._fromDom(xmlNode) )
        return xmlList
        

class TypedList(UserList):
    """ A list implementation which stores XMLObjects all of the same type.

    """
    
    def __init__(self, xmlList):
        UserList.__init__(self, [])
        self._xmlList = xmlList

    def _checkItem(self, it):
        itemType = None
        if not isinstance(it, XMLObject):
            itemType = type(it)
        elif self._xmlList.getItemType() != it.getName():
            itemType = it.getName()
        if itemType:
            raise TypeError("""\
%s type required for %s. Got %s instead""" % (`self._xmlList.getItemType()`,
                                              `self._xmlList.getName()`,
                                              `itemType`))

    def append(self, item):
        self._checkItem(item)
        UserList.append(self, item)

    def insert(self, index, item):
        self._checkItem(item)
        UserList.insert(self,index, item)

        
__all__ = [ 'StringAttribute', 'IntegerAttribute','ListNode',
            'TextNode', 'ItemNode', 'RequiredNodeError' ]

==> XMLObject-0.0.2.tar.gz: XMLObject-0.0.2/XMLObject/Node.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF Licence (see COPYING)

from UserList import UserList
from types import UnicodeType
import classregistry

from main import XMLObject

def encode(v,enc = "iso-8859-1"):
    if isinstance(v, UnicodeType):
        v = v.encode(enc)
    return v


class RequiredNodeError(Exception):

    def __init__(self, name):
        self._msg = "'%s' attribute is not mandatory" % name

    def __str__(self):
        return self._msg

    
class XONode(object):
    """ This is the base class for XMLObject's content. Each node has:

        - a `type` (string)
        - an `itemType` (string) : used by ListNode so that it remembers
          what kind of items it has to store.
        - a `default` value
        - a `facultative` boolean switch

        For each of these properties, there are getters and setters.
        * `xmlrepr` is the method responsible of giving a string representing
          the node. User defined nodes should override it.
        * `getValueFromDom` is used to build a Node given its DOM tree.
    """

    def __init__(self, type='', itemType='', default=None, facultative=False):
        self.setType(type)
        self.setItemType(itemType)
        self.setValue(default)
        self.setName('')
        self.setFacultative(facultative)

    def setFacultative(self, val):
        self._facultative = val

    def isFacultative(self):
        return self._facultative

    def setItemType(self, type):
        self._itemType = type

    def getItemType(self):
        return self._itemType
        
    def setName(self, name):
        self._name = name

    def getName(self):
        return self._name

    def setValue(self, value):
        self._value = value
        
    def getValue(self):
        return self._value

    def setType(self, type):
        self._type = type

    def getType(self):
        return self._type

    def getParentType(self):
        return self._parentType

    def setParentType(self, parentType):
        self._parentType = parentType

    def xmlrepr(self, indent=0):
        return ''

    def getValueFromDom(self, dom, attrName, **kw):
        return None

    def checkType(self, val):
        return True
    
class StringAttribute(XONode):
    """
        Suppose you have an XMLObject type like this one:

        class FooBar(XMLObject):
            glop = StringAttribute()

        FooBar(glop="Hey Yah").toXml() would output:

        <FooBar glop="Hey Yah" />
    """

    def __init__(self, default='', facultative=False):
        if facultative:
            default = None
        XONode.__init__(self, type='StringAttribute',
                        default=default, facultative=facultative)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return '%s="%s"' % (self.getName(),val)
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        xmlAttrNode = dom.getAttributeNode(attrName)
        try:
            value = encode(xmlAttrNode.value)
        except AttributeError:
            if not self.isFacultative():
                raise RequiredNodeError(self.getName())
            else:
                value = None
        return value

    def checkType(self, val):
        if self.isFacultative() and val is None:
            return True
        else:
            return type(val) == type('')

class IntegerAttribute(XONode):
    """

    """
    
    def __init__(self, default=0, facultative=False):
        if facultative:
            default = None
        XONode.__init__(self, type='IntegerAttribute',
                        default=default, facultative=facultative)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return '%s="%d"' % (self.getName(), val)
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        xmlAttrNode = dom.getAttributeNode(attrName)
        if not self.isFacultative() and not xmlAttrNode:
            raise RequiredNodeError(self.getName())
        if not xmlAttrNode:
            value = None
        else:
            value = int(encode(xmlAttrNode.value))
        return value

    def checkType(self, val):
        if self.isFacultative() and val is None:
            return True
        else:
            return type(val) == type(0)

class TextNode(XONode):
    """ Example class:

        class Foo(XMLObject):
            text = TextNode()

        f = Foo(text = 'Kind of fun')
        print f.toXml(headers=False)
        
        Example XML output:

        <Foo>
            <text>
                object value as a string
            </text>
        </Foo>
    """

    def __init__(self, default = '', facultative=False):
        XONode.__init__(self, type = 'Text', facultative=facultative,
                        default = default)

    def xmlrepr(self, indent=0):
        return ' ' * indent + str(self.getValue()) + '\n'

    def getValueFromDom(self, dom, attrName, **kw):
        value = ''
        for node in dom.childNodes:
            if node.nodeType == node.TEXT_NODE:
                value += node.data
        return encode(value.strip())

class ItemNode(XONode):
    """ Class used to refer to an existing XMLObject type.
        Suppose you have an XMLObject type like this one:

        class FooBar(XMLObject):
            glop = StringAttribute()

        You can then use it in other XMLObjects:

        class Blah(XMLObject):
            fb = ItemNode('FooBar')

        blah = Blah( fb = FooBar(glop = 'HelloWorld') )

        blah.toXml() would output:

            <Blah>
                 <FooBar glop="HelloWorld" />

            </Blah>
    """

    def __init__(self, itemType, default=None, facultative=False):
        XONode.__init__(self, type='Item', itemType=itemType,
                        facultative=facultative, default=default)

    def xmlrepr(self, indent=0):
        val = self.getValue()
        if val is not None:
            return val.toXml(indent=indent,headers=0) + '\n'
        else:
            return ''

    def getValueFromDom(self, dom, attrName, **kw):
        itemTypeName = self.getItemType()
        parentTypeName = self.getParentType()
        xmlObject = classregistry.registry(kw['registry']).getClass(itemTypeName)()
        candidates = dom.getElementsByTagName(itemTypeName)
        if len(candidates) == 0 and not self.isFacultative():
            raise RequiredNodeError(self.getName())
        value = None
        for candidate in candidates:
            if candidate.parentNode.nodeName == parentTypeName:
                value = xmlObject._fromDom(candidate)
                break
        return value

class ListNode(XONode):
    """


    """
    
    def __init__(self, itemType):
        XONode.__init__(self, type='List', itemType=itemType,
                        default=TypedList(self))
        
    def xmlrepr(self, indent=0):
        result = ''
        for node in self.getValue():
            result +=  node.toXml(indent=indent,headers=0) + '\n'
        return result

    def getValueFromDom(self, dom, attrName, **kw):
        xmlList = TypedList(self)
        itemTypeName = self.getItemType()
        parentTypeName = self.getParentType()
        klass = classregistry.registry(kw['registry']).getClass(itemTypeName)
        for xmlNode in dom.getElementsByTagName(itemTypeName):
            if xmlNode.parentNode.nodeName == parentTypeName:
                xmlObject = klass()
                xmlList.append( xmlObject._fromDom(xmlNode) )
        return xmlList
        

class TypedList(UserList):
    """ A list implementation which stores XMLObjects all of the same type.

    """
    
    def __init__(self, xmlList):
        UserList.__init__(self, [])
        self._xmlList = xmlList

    def _checkItem(self, it):
        itemType = None
        if not isinstance(it, XMLObject):
            itemType = type(it)
        elif self._xmlList.getItemType() != it.getName():
            itemType = it.getName()
        if itemType:
            raise TypeError("""\
%s type required for %s. Got %s instead""" % (`self._xmlList.getItemType()`,
                                              `self._xmlList.getName()`,
                                              `itemType`))

    def append(self, item):
        self._checkItem(item)
        UserList.append(self, item)

    def insert(self, index, item):
        self._checkItem(item)
        UserList.insert(self,index, item)

        
__all__ = [ 'StringAttribute', 'IntegerAttribute','ListNode',
            'TextNode', 'ItemNode', 'RequiredNodeError' ]
